<?php


namespace Drupal\niobi_app\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\niobi_app\Entity\NiobiApplicationWorkflow;
use Drupal\niobi_app\Entity\NiobiApplicationWorkflowInterface;
use Drupal\niobi_app\Entity\NiobiApplication;
use Drupal\niobi_app\Entity\NiobiApplicationInterface;
use Drupal\niobi_message\Plugin\task\Bundle\NotificationTask;
use Drupal\task\Entity\Task;
use Drupal\task\Entity\TaskInterface;
use Drupal\task\TaskUtilities;
use Drupal\niobi_app\Plugin\task\Action\DeclineReview;
use Drupal\niobi_app\Plugin\task\Action\DismissAsConflict;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Access\AccessResult;


/**
 * Provides the route controller for niobi_group.
 *
 */
class NiobiAppController extends ControllerBase {

  /**
   * @return RedirectResponse
   */
  public function redirectToPrevious() {
    // Return to where we came from
    $server = Request::createFromGlobals()->server;
    $return = $server->get('HTTP_REFERER');
    $http = $server->get('HTTPS') ? 'https://' : 'http://';
    $domain = $http . $server->get('SERVER_NAME');
    $port = ':' . $server->get('SERVER_PORT');
    $return = str_replace($domain, '', $return);
    $return = str_replace($port, '', $return);
    $url = Url::fromUserInput($return ? $return : '/');

    $response = new RedirectResponse($url->toString());
    $request = \Drupal::request();

    // Save the session so things like messages get saved.
    $request->getSession()->save();
    $response->prepare($request);

    return $response;
  }

  /**
   * @param NiobiApplicationWorkflow $niobi_application_workflow
   * @return array
   */
  public function launchChecklists($niobi_application_workflow) {
    return ['#type' => 'markup', '#markup' => ''];
  }

  /**
   * {@inheritdoc}
   */
  public function createApplicationEntity(NiobiApplicationWorkflow $application_workflow, $status = 'application') {
    $data = [
      'user_id' => \Drupal::currentUser()->id(),
      'name' => \Drupal::currentUser()->getAccountName() . ' - ' . $application_workflow->label(),
      'field_application_status' => $status,
      'field_application_workflow' => $application_workflow->id(),
      'field_current_stage' => $application_workflow->getFirstStage()->id(),
    ];
    $application = NiobiApplication::create($data);
    $application->save();
    return $application;
  }

  /**
   * {@inheritdoc}
   */
  public function createApplication(NiobiApplicationWorkflow $niobi_application_workflow) {
    $application = $this->createApplicationEntity($niobi_application_workflow, 'application');
    $application_form = $niobi_application_workflow->getFirstStage()->getApplicationForm();
    $params = [
      'niobi_form' => $application_form->id()
    ];
    $options = [
      'query' => [
        'application_id' => $application->get('uuid')->value,
      ]
    ];
    $url = Url::fromRoute('entity.niobi_form.canonical', $params, $options);
    $response = new RedirectResponse($url->toString());
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function createNomination(NiobiApplicationWorkflow $niobi_application_workflow) {
    $nomination = $this->createApplicationEntity($niobi_application_workflow, 'nomination');
    $nomination_form = $niobi_application_workflow->getFirstStage()->getNominationForm();
    $params = [
      'niobi_form' => $nomination_form->id()
    ];
    $options = [
      'query' => [
        'application_id' => $nomination->get('uuid')->value,
      ]
    ];
    $url = Url::fromRoute('entity.niobi_form.canonical', $params, $options);
    $response = new RedirectResponse($url->toString());
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function acceptNomination($uuid) {
    $check = FALSE;
    if (is_string($uuid) && (preg_match('/^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/', $uuid) === 1)) {
      $check = TRUE;
    }
    if ($check) {
      $a = \Drupal::entityTypeManager()->getStorage('niobi_application')->loadByProperties(['uuid' => $uuid]);
      $id = array_keys($a)[0];
      if ($id) {
        if (\Drupal::currentUser()->isAnonymous()) {
          $params = ['destination' => Url::fromRoute('niobi_application.accept_nomination', ['uuid' => $uuid])->getInternalPath()];
          $url = Url::fromRoute('user.login',[],$params);
          $response = new RedirectResponse($url->toString());
          return $response;
        }
        // User is logged in. Determine if they have access.
        if (\Drupal::currentUser()->hasPermission('edit niobi application entities')) {
          $application = NiobiApplication::load($id);
          $workflow = $application->getApplicationWorkflow();
          $application->set('field_application_status', 'application');
          $application->set('user_id', \Drupal::currentUser()->id());
          $application->save();
          $params = [
            'niobi_application_workflow' => $workflow->id()
          ];
          $url = Url::fromRoute('entity.niobi_application_workflow.canonical', $params);
          $response = new RedirectResponse($url->toString());
          return $response;
        }
        else {
          return AccessResult::forbidden();
        }
      }
    }

    // If we didn't get the redirect, return an error message.
    return ['#type' => 'markup' , '#markup' => '<h2>Invalid Application Code</h2>'];
  }

  /**
   * {@inheritdoc}
   */
  public function declineNomination($uuid) {
    $check = FALSE;
    if (is_string($uuid) && (preg_match('/^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/', $uuid) === 1)) {
      $check = TRUE;
    }
    if ($check) {
      $a = \Drupal::entityTypeManager()->getStorage('niobi_application')->loadByProperties(['uuid' => $uuid]);
      $id = array_keys($a)[0];
      if ($id) {
        $application = NiobiApplication::load($id);
        $workflow = $application->getApplicationWorkflow();
        $application->set('field_application_status', 'complete');
        $application->save();
        $params = [
          'niobi_application_workflow' => $workflow->id()
        ];
        $url = Url::fromRoute('entity.niobi_application_workflow.canonical', $params);
        $response = new RedirectResponse($url->toString());
        return $response;
      }
    }

    // If we didn't get the redirect, return an error message.
    return ['#type' => 'markup' , '#markup' => '<h2>Invalid Application Code</h2>'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitNomination(NiobiApplicationWorkflowInterface $niobi_application_workflow, NiobiApplicationInterface $niobi_application) {
    $niobi_application->set('field_application_status', 'nomination_claim');
    $niobi_application->save();
    $params = [
      'niobi_application_workflow' => $niobi_application_workflow->id()
    ];
    $url = Url::fromRoute('entity.niobi_application_workflow.canonical', $params);
    $response = new RedirectResponse($url->toString());
    return $response;
  }

  /**
   * Submit an application for review
   * @param NiobiApplicationWorkflowInterface $niobi_application_workflow
   * @param NiobiApplicationInterface $niobi_application
   * @return RedirectResponse
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function submitApplication(NiobiApplicationWorkflowInterface $niobi_application_workflow, NiobiApplicationInterface $niobi_application) {
    $niobi_application = NiobiApplication::load($niobi_application->id());
    $stage = $niobi_application->getCurrentApplicationStage();

    // Run any trigger tasks
    $query = \Drupal::entityQuery('task');
    $query->condition('type', 'niobi_notification_task');
    $query->condition('field_trigger_type', 'niobi_app_submit_application');
    $query->condition('status', 'active');
    $result = $query->execute();

    $tasks = Task::loadMultiple($result);

    foreach ($tasks as $task) {
      NotificationTask::triggerTask($task);
    }

    // Run review/decision logic.
    /* @var $stage \Drupal\niobi_app\Entity\NiobiApplicationWorkflowStage */
    $review_forms = $stage->getReviewForms(TRUE);

    if (!empty($review_forms)) {
      // Run auto-assignment
      NiobiAppController::runAutoAssignment($niobi_application);

      // Set as "in review" .
      $niobi_application->set('field_application_status', 'review');
      $niobi_application->save();
    }
    else {
      $this->moveToDecision($niobi_application);
    }


    // Redirect to application dashboard.
    $params = [
      'niobi_application_workflow' => $niobi_application_workflow->id()
    ];
    $url = Url::fromRoute('entity.niobi_application_workflow.canonical', $params);
    $response = new RedirectResponse($url->toString());
    return $response;
  }

  public static function runAutoAssignment(NiobiApplication $niobi_application) {
    // Run auto-assignments.
    $stage = $niobi_application->getCurrentApplicationStage();

    $auto_assign_info = $stage->getReviewAutoAssignmentInfo($niobi_application);

    if ($auto_assign_info) {
      // If there is no maximum, assign to everyone.
      if (!$auto_assign_info['max'] || $auto_assign_info['pool_count'] <= $auto_assign_info['max']) {
        $reviewers = $auto_assign_info['pool']->getAutoAssignmentReviewers($niobi_application, $stage);
        foreach ($reviewers as $id => $reviewer) {
          // Get data for application owner
          $label_info = [
            '@reviewer' => $reviewer->label(),
            '@app' => $niobi_application->label(),
          ];
          $data = [
            'type' => 'application_review_task',
            'status' => 'active',
            'name' => t('Assign @reviewer to @app', $label_info),
            'assigned_by_type' => 'system',
            'assigned_by' => '0',
            'assigned_to' => $id,
            'assigned_to_type' => 'user',
            'field_application' => $niobi_application->id(),
            'field_review_form' => $auto_assign_info['form']->id(),
          ];
          TaskUtilities::createTask($data);
        }
      }
    }
  }

  /**
   * @param TaskInterface $task
   * @return \Drupal\Core\Access\AccessResultForbidden|RedirectResponse
   */
  public function DeclineReviewAssignment(TaskInterface $task) {
    $assignee = $task->get('assigned_to')->getValue();
    if (isset($assignee[0]['target_id'])) {
      if (\Drupal::currentUser()->id() === $assignee[0]['target_id']) {
        DeclineReview::doAction($task);
        return $this->redirectToPrevious();
      }
      else {
        \Drupal::messenger()->addError('You do not have access to this task, or the task has missing/incorrect information.');
        return AccessResult::forbidden();
      }
    }
    else {
      \Drupal::messenger()->addError('This review assignment was already dismissed, or has missing information.');
      return $this->redirectToPrevious();
    }
  }

  /**
   * @param TaskInterface $task
   * @return \Drupal\Core\Access\AccessResultForbidden|RedirectResponse
   */
  public function DeclineReviewAssignmentAndMarkCOI(TaskInterface $task) {
    $assignee = $task->get('assigned_to')->getValue();
    if (isset($assignee[0]['target_id'])) {
      if (\Drupal::currentUser()->id() === $assignee[0]['target_id']) {
        DismissAsConflict::doAction($task);
        return $this->redirectToPrevious();
      }
      else {
        \Drupal::messenger()->addError('You do not have access to this task, or the task has missing/incorrect information.');
        return AccessResult::forbidden();
      }
    }
    else {
      \Drupal::messenger()->addError('This review assignment was already dismissed, or has missing information.');
      return $this->redirectToPrevious();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function moveToDecision(NiobiApplicationInterface $niobi_application) {
    $application = NiobiApplication::load($niobi_application->id());
    $stage = $application->getCurrentApplicationStage();
    /* @var $stage \Drupal\niobi_app\Entity\NiobiApplicationWorkflowStage */
    $auto_decision = $stage->getAutoDecision();
    if ($auto_decision) {
      $auto_decision['class']::autoDecision($application);
    }
    else {
      $application->set('field_application_status', 'decision')->save();
    }
    return $this->redirectToPrevious();
  }

}
