<?php
/**
 * @file
 * Contains \Drupal\niobi_app\Form\NiobiAppReviewAssignmentForm.
 */

namespace Drupal\niobi_app\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\task\TaskUtilities;
use Drupal\user\Entity\User;
use Drupal\niobi_app\Entity\NiobiApplication;

class NiobiAppReviewAssignmentForm extends FormBase {
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'amcs_section_response_add_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $niobi_application = NULL) {
    /* @var $niobi_application \Drupal\niobi_app\Entity\NiobiApplication */
    if ($niobi_application) {
      $workflow_stage = $niobi_application->getCurrentApplicationStage();
      /* @var $workflow \Drupal\niobi_app\Entity\NiobiApplicationWorkflowStage */
      $pools = $workflow_stage->getReviewerPools();
      $form['application_label'] = [
        '#type' => 'html_tag',
        '#tag' => 'h3',
        '#value' => t('Application: %app', ['%app' => $niobi_application->label()])
      ];
      $form['application'] = [
        '#type' => 'hidden',
        '#value' => $niobi_application->id(),
      ];
      $form['workflow_stage'] = [
        '#type' => 'hidden',
        '#value' => $workflow_stage->id(),
      ];
      $review_forms = $workflow_stage->getReviewFormsAsNames();
      $rfkeys = array_keys($review_forms);
      $form['review_form'] = [
        '#title' => 'Select Review Form',
        '#type' => 'select',
        '#options' => $review_forms,
        '#required' => TRUE,
        '#empty_option' => !empty($review_forms) ? '- Select -' : 'No review forms have been attached to this workflow stage.',
        '#default_value' => count($review_forms) === 1 ? array_shift($rfkeys) : NULL,
      ];
      $form['reviewer'] = [
        '#title' => 'Select Reviewer',
        '#type' => 'select',
        '#options' => [],
        '#required' => TRUE,
        '#empty_option' => !empty($pools) ? '- Select -' : 'No pools have been attached to this workflow stage.',
      ];
      foreach ($pools as $pool) {
        /* @var $pool \Drupal\niobi_app\Entity\NiobiApplicationReviewerPool */
        if (!empty($pool->getReviewersAsNames($niobi_application, $workflow_stage)) ) {
          $form['reviewer']['#options'][$pool->label()] = $pool->getReviewersAsNames($niobi_application, $workflow_stage);
        }
        else {
          $form['reviewer']['#options'][$pool->label()] = ['none' => t('This pool is empty, or all reviewers are excluded.')];
        }

      }
      if (count($pools) === 1) {
        $pool = array_shift($pools);
        $names = $pool->getReviewersAsNames($niobi_application, $workflow_stage);
        if (count($names) === 1) {
          $form['reviewer']['#default_value'] = array_shift(array_keys($names));
        }
        else if (count($names) === 0) {
          $form['reviewer']['#default_value'] = ['none' => t('This pool is empty, or all reviewers are excluded.')];
        }
      }
      $form['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
        '#button_type' => 'primary',
      ];
    }


    return $form;
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $vals = $form_state->getValues();

    $reviewer = User::load($vals['reviewer']);
    $niobi_application = NiobiApplication::load($vals['application']);
    // Get data for application owner
    $label_info = [
      '@reviewer' => $reviewer->label(),
      '@app' => $niobi_application->label(),
    ];
    $data = [
      'type' => 'application_review_task',
      'status' => 'active',
      'name' => t('Assign @reviewer to @app', $label_info),
      'assigned_by_type' => 'system',
      'assigned_by' => '0',
      'assigned_to' => $reviewer->id(),
      'assigned_to_type' => 'user',
      'field_application' => $niobi_application->id(),
      'field_review_form' => $vals['review_form'],
    ];
    $task = TaskUtilities::createTask($data);
    if (!empty($task)) {
      \Drupal::messenger()->addMessage('Review assignment created');
    }
  }
}